/*
			Copyright (c) 1994 by
			Advanced Visual Systems Inc.
			All Rights Reserved

	This software comprises unpublished confidential information of
	Advanced Visual Systems Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.

	This file is under Perforce control
	$Id: //depot/express/fcs992/user/add_num.c#1 $
*/

/******************************************************************************
 *
 * These functions are invoked by the elements of the same name
 * defined in the file express/v/user.v (and linked together
 * during Express initialization by a function "user_func_init" 
 * located in express/user/user.c
 *
 *****************************************************************************/

#include <avs/err.h>
#include <avs/om.h>

/******************************************************************************
 * Add two numbers
 *****************************************************************************/
int
USERadd_num(elem_id, event_mask, seq_num)
    OMobj_id elem_id;
    OMevent_mask event_mask;
    int seq_num;
{
    OMobj_id src_1_id, src_2_id, res_id;
    double src_1, src_2, res;

    /*
     * First we look up the element id's for each of our sub-elements.
     * If the element id is null, we use an error macro to print an
     * error and then return 0 to indicate a failure.
     */
    src_1_id = OMfind_subobj(elem_id, OMstr_to_name("src_1"), OM_OBJ_RD);
    if (OMis_null_obj(src_1_id)) {
	ERRerror("add_num", 1, ERR_ORIG, "can't find src_1"); 
	return(0);	
    }

    src_2_id = OMfind_subobj(elem_id, OMstr_to_name("src_2"), OM_OBJ_RD);
    if (OMis_null_obj(src_2_id)) {
	ERRerror("add_num", 1, ERR_ORIG, "can't find src_2"); 
	return(0);	
    }

    res_id = OMfind_subobj(elem_id, OMstr_to_name("res"), OM_OBJ_RW);
    if (OMis_null_obj(res_id)) {
	ERRerror("add_num", 1, ERR_ORIG, "can't find res"); 
	return(0);	
    }

    /*
     * Next, we get the value of the source subelements 
     */
    OMget_real_val(src_1_id, &src_1);
    OMget_real_val(src_2_id, &src_2);

    /*
     * Next, we add the two source values
     */

    res = src_1 + src_2;

    /* 
     * Finally, we set the subelement value of the result element 
     * and return 1 to indicate success.
     */

    OMset_real_val(res_id, res);

    return(1);
}

/******************************************************************************
 * Subtract two numbers
 *****************************************************************************/
int
USERsubtract_num(elem_id, event_mask, seq_num)
    OMobj_id elem_id;
    OMevent_mask event_mask;
    int seq_num;
{
    double src_1, src_2, res;

    /*
     * First, we get the value of the source subelements.
     * "OMget_name_real_val" takes the parent element as its
     * first argument, and the target element as its second.
     * Here, we use the function OMstr_to_name to lookup the
     * element id for the target.
     */
    OMget_name_real_val(elem_id, OMstr_to_name("src_1"), &src_1);
    OMget_name_real_val(elem_id, OMstr_to_name("src_2"), &src_2);

    /*
     * Next, we add the two source values (the easy part)
     */
    res = src_1 - src_2;

    /* 
     * Finally, we set the subelement value of the result element 
     * and return 1 to indicate success.
     */
    OMset_name_real_val(elem_id, OMstr_to_name("res"), res);

    return(1);
}
